@@ -276,12 +276,16 @@ save_object (gpointer key, gpointer value, gpointer vcalcomp)
 	icalcomponent *icalcomp;
 	GList *l;
 
-	icalcomp = icalcomponent_new_clone (ecomp->icomp);
-	icalcomponent_add_component (vcalcomp, icalcomp);
+	if (ecomp->icomp) {
+		icalcomp = icalcomponent_new_clone (ecomp->icomp);
+		icalcomponent_add_component (vcalcomp, icalcomp);
+	}
 
 	for (l = ecomp->instances; l; l = l->next) {
-		icalcomp = icalcomponent_new_clone (l->data);
-		icalcomponent_add_component (vcalcomp, icalcomp);
+		if (l->data) {
+			icalcomp = icalcomponent_new_clone (l->data);
+			icalcomponent_add_component (vcalcomp, icalcomp);
+		}
 	}
 }
 
@@ -300,7 +304,7 @@ timeout_save_cache (gpointer user_data)
 	vcalcomp = e_cal_util_new_top_level ();
 	g_hash_table_foreach (cbex->priv->timezones, save_timezone, vcalcomp);
 	g_hash_table_foreach (cbex->priv->objects, save_object, vcalcomp);
-	data = icalcomponent_as_ical_string (vcalcomp);
+	data = icalcomponent_as_ical_string_r (vcalcomp);
 	icalcomponent_free (vcalcomp);
 
 	tmpfile = g_strdup_printf ("%s~", cbex->priv->object_cache_file);
@@ -427,6 +431,7 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 	if (!cbex->folder) {
 		ESource *source;
 		const char *foreign;
+		ExchangeHierarchy *hier_to_rescan = NULL;
 		/* FIXME: theoretically we should create it if
 		 * only_if_exists is FALSE.
 		 */
@@ -443,21 +448,30 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 			email = split_path [0];
 
 			exchange_account_scan_foreign_hierarchy (cbex->account, email);
+
 			cbex->folder = exchange_account_get_folder (cbex->account, uristr);
+			if (!cbex->folder) {
+				/* Folder is not known at the moment, thus try to rescan foreign
+				   folder, just in case we didn't scan it fully yet. */
+				hier_to_rescan = exchange_account_get_hierarchy_by_email (cbex->account, email);
+			}
 
 			g_strfreev (split_path);
 		} else {
 			/* Rescan to see if this is any new calendar */
-		        ExchangeHierarchy *hier;
-
-			hier = exchange_account_get_hierarchy_by_type (cbex->account, EXCHANGE_HIERARCHY_PERSONAL);
-			if (!hier)
+			hier_to_rescan = exchange_account_get_hierarchy_by_type (cbex->account, EXCHANGE_HIERARCHY_PERSONAL);
+			if (!hier_to_rescan) {
+				g_mutex_unlock (cbex->priv->open_lock);
 				return GNOME_Evolution_Calendar_RepositoryOffline;
-	       	       	g_object_ref (hier->toplevel);
-	       	       	e_folder_exchange_set_rescan_tree (hier->toplevel, TRUE);
-	               	exchange_hierarchy_scan_subtree (hier, hier->toplevel, ONLINE_MODE);
-	               	e_folder_exchange_set_rescan_tree (hier->toplevel, FALSE);
-	               	g_object_unref (hier->toplevel);
+			}
+		}
+
+		if (hier_to_rescan) {
+	       	       	g_object_ref (hier_to_rescan->toplevel);
+	       	       	e_folder_exchange_set_rescan_tree (hier_to_rescan->toplevel, TRUE);
+	               	exchange_hierarchy_scan_subtree (hier_to_rescan, hier_to_rescan->toplevel, ONLINE_MODE);
+	               	e_folder_exchange_set_rescan_tree (hier_to_rescan->toplevel, FALSE);
+	               	g_object_unref (hier_to_rescan->toplevel);
 
 	               	cbex->folder = exchange_account_get_folder (cbex->account, uristr);
 		 	if (!cbex->folder)
@@ -593,6 +607,29 @@ find_instance (ECalBackendExchange *cbex, ECalBackendExchangeComponent *ecomp, c
 	return found;
 }
 
+/**
+ * e_cal_backend_exchange_ensure_utc_zone:
+ * Makes sure the given icaltimetype is in UTC, because
+ * RFC 2445 says CREATED/DTSTAMP/LAST-MODIFIED always in UTC.
+ * @param cb ECalbackendExchage descendant.
+ * @param itt What to convert to UTC, if required.
+ **/
+void
+e_cal_backend_exchange_ensure_utc_zone (ECalBackend *cb, struct icaltimetype *itt)
+{
+	g_return_if_fail (cb != NULL);
+	g_return_if_fail (itt != NULL);
+
+	/* RFC 2445 - CREATED/DTSTAMP/LAST-MODIFIED always in UTC */
+	if (!icaltime_is_null_time (*itt) && !icaltime_is_utc (*itt)) {
+		if (!itt->zone)
+			icaltime_set_timezone (itt, e_cal_backend_internal_get_default_timezone (cb));
+
+		icaltimezone_convert_time (itt, (icaltimezone*) icaltime_get_timezone (*itt), icaltimezone_get_utc_timezone ());
+		icaltime_set_timezone (itt, icaltimezone_get_utc_timezone ());
+	}
+}
+
 gboolean
 e_cal_backend_exchange_in_cache (ECalBackendExchange *cbex,
 				 const char          *uid,
@@ -645,8 +682,8 @@ uncache (gpointer key, gpointer value, gpointer data)
 		char *str = NULL;
 		/* FIXME somehow the query does not match with the component in some cases, so user needs to press a
 		   clear to get rid of the component from the view in that case*/
-		str = icalcomponent_as_ical_string (ecomp->icomp);
-		e_cal_backend_notify_object_removed (backend, id, icalcomponent_as_ical_string (ecomp->icomp)
+		str = icalcomponent_as_ical_string_r (ecomp->icomp);
+		e_cal_backend_notify_object_removed (backend, id, icalcomponent_as_ical_string_r (ecomp->icomp)
 				, NULL);
 		g_free (str);
 	}
@@ -805,7 +842,8 @@ e_cal_backend_exchange_modify_object (ECalBackendExchange *cbex,
 		return FALSE;
 
 	if (mod == CALOBJ_MOD_ALL || icaltime_is_null_time (rid) || discard_detached) {
-		icalcomponent_free (ecomp->icomp);
+		if (ecomp->icomp)
+			icalcomponent_free (ecomp->icomp);
 		ecomp->icomp = icalcomponent_new_clone (comp);
 		if (discard_detached && !icaltime_is_null_time (rid))
 			discard_detached_instance (ecomp, rid);
@@ -948,7 +986,7 @@ get_object (ECalBackendSync *backend, EDataCal *cal,
 		}
 
 		if (inst_found) {
-			*object = icalcomponent_as_ical_string (l->data);
+			*object = icalcomponent_as_ical_string_r (l->data);
 		} else {
 			/* Instance is not found. Send the master object instead */
 			if (ecomp->icomp) {
@@ -962,7 +1000,7 @@ get_object (ECalBackendSync *backend, EDataCal *cal,
 					return GNOME_Evolution_Calendar_ObjectNotFound;
 				}
 
-				*object = icalcomponent_as_ical_string (new_inst);
+				*object = icalcomponent_as_ical_string_r (new_inst);
 				icalcomponent_free (new_inst);
 			} else {
 				/* Oops. No instance and no master as well !! */
@@ -985,11 +1023,11 @@ get_object (ECalBackendSync *backend, EDataCal *cal,
 					icalcomponent_new_clone (ecomp->icomp));
 			g_list_foreach (ecomp->instances, (GFunc) add_instances_to_vcal, vcalcomp);
 
-			*object = icalcomponent_as_ical_string (vcalcomp);
+			*object = icalcomponent_as_ical_string_r (vcalcomp);
 			icalcomponent_free (vcalcomp);
-		} else {
+		} else if (ecomp->icomp) {
 			/* There are no detached instances. Send only the master object */
-			*object = icalcomponent_as_ical_string (ecomp->icomp);
+			*object = icalcomponent_as_ical_string_r (ecomp->icomp);
 		}
 	}
 	g_mutex_unlock (cbex->priv->cache_lock);
@@ -1082,7 +1120,7 @@ get_default_object (ECalBackendSync *backend, EDataCal *cal, char **object)
 	d(printf("ecbe_get_default_object(%p, %p)\n", backend, cal));
 
 	comp = e_cal_util_new_component (e_cal_backend_get_kind (E_CAL_BACKEND (backend)));
-	ical_obj = icalcomponent_as_ical_string (comp);
+	ical_obj = icalcomponent_as_ical_string_r (comp);
 	*object = ical_obj;
 
 	icalcomponent_free (comp);
@@ -1211,7 +1249,7 @@ get_timezone (ECalBackendSync *backend, EDataCal *cal,
 	if (!vtzcomp)
 		return GNOME_Evolution_Calendar_OtherError;
 
-	ical_obj = icalcomponent_as_ical_string (vtzcomp);
+	ical_obj = icalcomponent_as_ical_string_r (vtzcomp);
 	*object = ical_obj;
 
 	return  GNOME_Evolution_Calendar_Success;
@@ -1403,7 +1441,6 @@ set_mode (ECalBackend *backend, CalMode mode)
 			priv->read_only = FALSE;
 			priv->mode = CAL_MODE_REMOTE;
 
-			e_cal_backend_notify_readonly (backend, priv->read_only);
 			if (is_loaded (backend) && re_open)
 				e_cal_backend_notify_auth_required(backend);
 			break;
@@ -1415,7 +1452,6 @@ set_mode (ECalBackend *backend, CalMode mode)
 			e_cal_backend_notify_mode (backend,
 				GNOME_Evolution_Calendar_CalListener_MODE_SET,
 				GNOME_Evolution_Calendar_MODE_LOCAL);
-			e_cal_backend_notify_readonly (backend, priv->read_only);
 			break;
 
 	default :
@@ -1600,7 +1636,6 @@ check_change_type (gpointer key, gpointer value, gpointer data)
 	*/
 	if (!ecomp)
 		return;
-	icomp = ecomp->icomp;
 	l = ecomp->instances;
 	for (icomp = ecomp->icomp; l; icomp = l->data, l = l->next) {
 		if (!icomp)
@@ -1792,7 +1827,7 @@ process_delegated_cal_object (icalcomponent *icalcomp, const char *delegator_nam
 		const char *organizer;
 		char *text = NULL;
 
-		organizer = icalproperty_get_value_as_string (prop);
+		organizer = icalproperty_get_value_as_string_r (prop);
 		if (organizer) {
 			if (!g_ascii_strncasecmp (organizer, "mailto:", 7))
 				text = g_strdup (organizer+7);
@@ -1817,7 +1852,7 @@ process_delegated_cal_object (icalcomponent *icalcomp, const char *delegator_nam
 		const char *attendee;
 		char *text = NULL;
 
-		attendee = icalproperty_get_value_as_string (prop);
+		attendee = icalproperty_get_value_as_string_r (prop);
 		if (!attendee)
 			continue;
 
@@ -2094,7 +2129,8 @@ free_exchange_comp (gpointer value)
 	g_free (ecomp->href);
 	g_free (ecomp->lastmod);
 
-	icalcomponent_free (ecomp->icomp);
+	if (ecomp->icomp)
+		icalcomponent_free (ecomp->icomp);
 
 	for (inst = ecomp->instances; inst; inst = inst->next)
 		icalcomponent_free (inst->data);
