@@ -27,8 +27,8 @@
 #include <string.h>
 #include <sys/stat.h>
 
+#include <glib/gi18n-lib.h>
 #include <libedataserver/e-data-server-util.h>
-#include <camel/camel-i18n.h>
 
 #include "camel-exchange-folder.h"
 #include "camel-exchange-search.h"
@@ -50,10 +50,6 @@
 
 static CamelOfflineFolderClass *parent_class = NULL;
 
-static CamelProperty exchange_property_list[] = {
-	{ CAMEL_EXCHANGE_FOLDER_CHECK_FOLDER, "check_folder", N_("Always check for new mail in this folder") },
-};
-
 /* Returns the class for a CamelFolder */
 #define CF_CLASS(so) CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 
@@ -67,6 +63,10 @@ static CamelMimeMessage *get_message         (CamelFolder *folder,
 static GPtrArray      *search_by_expression  (CamelFolder *folder,
 					      const char *exp,
 					      CamelException *ex);
+static guint32 	      count_by_expression  (CamelFolder *folder,
+					      const char *exp,
+					      CamelException *ex);
+
 static GPtrArray      *search_by_uids        (CamelFolder *folder,
 					      const char *expression,
 					      GPtrArray *uids,
@@ -85,9 +85,8 @@ static void   transfer_messages_the_hard_way (CamelFolder *source,
 					      CamelException *ex);
 static void refresh_info (CamelFolder *folder, CamelException *ex);
 static void exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex);
-
-static int exchange_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args);
-static int exchange_setv (CamelObject *object, CamelException *ex, CamelArgV *args);
+static char* get_filename (CamelFolder *folder, const char *uid, CamelException *ex);
+static gint cmp_uids (CamelFolder *folder, const char *uid1, const char *uid2);
 
 static void
 class_init (CamelFolderClass *camel_folder_class)
@@ -99,15 +98,16 @@ class_init (CamelFolderClass *camel_folder_class)
 	camel_folder_class->append_message = append_message;
 	camel_folder_class->get_message = get_message;
 	camel_folder_class->search_by_expression = search_by_expression;
+	camel_folder_class->count_by_expression = count_by_expression;
+
+	camel_folder_class->cmp_uids = cmp_uids;
 	camel_folder_class->search_by_uids = search_by_uids;
 	/* use the default function for the search_free */
 	/* camel_folder_class->search_free = search_free; */
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
 	camel_folder_class->refresh_info = refresh_info;
 	camel_folder_class->sync = exchange_sync;
-
- 	((CamelObjectClass *)camel_folder_class)->getv = exchange_getv;
-	((CamelObjectClass *)camel_folder_class)->setv = exchange_setv;
+	camel_folder_class->get_filename = get_filename;
 }
 
 #define CAMEL_EXCHANGE_SERVER_FLAGS \
@@ -149,8 +149,6 @@ camel_exchange_folder_get_type (void)
 	static CamelType camel_exchange_folder_type = CAMEL_INVALID_TYPE;
 
 	if (camel_exchange_folder_type == CAMEL_INVALID_TYPE) {
-		int i;
-
 		camel_exchange_folder_type = camel_type_register (
 			camel_offline_folder_get_type (),
 			"CamelExchangeFolder",
@@ -160,15 +158,12 @@ camel_exchange_folder_get_type (void)
 			NULL,
 			(CamelObjectInitFunc) init,
 			(CamelObjectFinalizeFunc) finalize );
-
-		/* only localize here, do not create GSList, we do not want to leak */
-		for (i = 0; i < sizeof (exchange_property_list)/sizeof (exchange_property_list [0]); i++)
-			exchange_property_list [i].description = _(exchange_property_list [i].description);
 	}
 
 	return camel_exchange_folder_type;
 }
 
+
 static void
 refresh_info (CamelFolder *folder, CamelException *ex)
 {
@@ -371,6 +366,14 @@ fix_broken_multipart_related (CamelMimePart *part)
 	}
 }
 
+static char*
+get_filename (CamelFolder *folder, const char *uid, CamelException *ex)
+{
+	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
+
+	return camel_data_cache_get_filename (exch->cache, "cache", uid, NULL);
+}
+
 static GByteArray *
 get_message_data (CamelFolder *folder, const char *uid, CamelException *ex)
 {
@@ -489,6 +492,31 @@ search_by_expression (CamelFolder *folder, const char *expression,
 	return matches;
 }
 
+static guint32
+count_by_expression (CamelFolder *folder, const char *expression,
+		      CamelException *ex)
+{
+	CamelFolderSearch *search;
+	guint32 matches;
+
+	search = camel_exchange_search_new ();
+	camel_folder_search_set_folder (search, folder);
+	matches = camel_folder_search_count (search, expression, ex);
+
+	camel_object_unref (CAMEL_OBJECT (search));
+
+	return matches;
+}
+
+static gint
+cmp_uids (CamelFolder *folder, const char *uid1, const char *uid2)
+{
+	g_return_val_if_fail (uid1 != NULL, 0);
+	g_return_val_if_fail (uid2 != NULL, 0);
+
+	return strcmp (uid1, uid2);
+}
+
 static GPtrArray *
 search_by_uids (CamelFolder *folder, const char *expression,
 		GPtrArray *uids, CamelException *ex)
@@ -722,7 +750,6 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	CamelFolderChangeInfo *changes;
 	CamelStream *stream;
 	CamelMimeMessage *msg;
-	guint32 unread_count, visible_count;
 
 	info = camel_folder_summary_uid (folder->summary, uid);
 	if (info) {
@@ -1121,78 +1148,3 @@ exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 	camel_folder_summary_save_to_db (folder->summary, ex);
 }
-
-static int
-exchange_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args)
-{
-	int i, count=0;
-	guint32 tag;
-
-	for (i = 0; i < args->argc; i++) {
-		CamelArgGet *arg = &args->argv[i];
-
-		tag = arg->tag;
-
-		switch (tag & CAMEL_ARG_TAG) {
-		case CAMEL_OBJECT_ARG_PERSISTENT_PROPERTIES:
-		case CAMEL_FOLDER_ARG_PROPERTIES: {
-			CamelArgGetV props;
-			int i;
-
-			props.argc = 1;
-			props.argv[0] = *arg;
-			((CamelObjectClass *)parent_class)->getv(object, ex, &props);
-
-			for (i = 0; i < sizeof (exchange_property_list)/sizeof (exchange_property_list [0]); i++)
-				*arg->ca_ptr = g_slist_append (*arg->ca_ptr, &exchange_property_list[i]);
-			break; }
-			/* exchange args */
-		case CAMEL_EXCHANGE_FOLDER_ARG_CHECK_FOLDER:
-			*arg->ca_int = ((CamelExchangeFolder *)object)->check_folder;
-			break;
-		default:
-			count++;
-			continue;
-		}
-
-		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
-	}
-
-	if (count)
-		return ((CamelObjectClass *)parent_class)->getv (object, ex, args);
-
-	return 0;
-}
-
-static int
-exchange_setv (CamelObject *object, CamelException *ex, CamelArgV *args)
-{
-	int save = 0;
-	int i;
-	guint32 tag;
-
-	for (i = 0; i < args->argc; i++) {
-		CamelArg *arg = &args->argv[i];
-
-		tag = arg->tag;
-
-		switch (tag & CAMEL_ARG_TAG) {
-		case CAMEL_EXCHANGE_FOLDER_ARG_CHECK_FOLDER:
-			if (((CamelExchangeFolder *)object)->check_folder != arg->ca_int) {
-				((CamelExchangeFolder *)object)->check_folder = arg->ca_int;
-				save = 1;
-			}
-			break;
-		default:
-			continue;
-		}
-
-		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
-	}
-
-	if (save)
-		camel_object_state_write (object);
-
-	return ((CamelObjectClass *)parent_class)->setv (object, ex, args);
-}
-
