@@ -26,13 +26,7 @@
 #include <utime.h>
 #include <string.h>
 
-#include <glib.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkprogressbar.h>
 #include <libedataserver/e-data-server-util.h>
 #include <e-util/e-folder-map.h>
 
@@ -80,19 +74,19 @@ setup_progress_dialog ()
 
 	gtk_label_set_line_wrap ((GtkLabel *) w, TRUE);
 	gtk_widget_show (w);
-	gtk_box_pack_start_defaults ((GtkBox *) vbox, w);
+	gtk_box_pack_start ((GtkBox *) vbox, w, TRUE, TRUE, 0);
 
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_widget_show (hbox);
-	gtk_box_pack_start_defaults ((GtkBox *) vbox, hbox);
+	gtk_box_pack_start ((GtkBox *) vbox, hbox, TRUE, TRUE, 0);
 
 	label = (GtkLabel *) gtk_label_new ("");
 	gtk_widget_show ((GtkWidget *) label);
-	gtk_box_pack_start_defaults ((GtkBox *) hbox, (GtkWidget *) label);
+	gtk_box_pack_start ((GtkBox *) hbox, (GtkWidget *) label, TRUE, TRUE, 0);
 
 	progress = (GtkProgressBar *) gtk_progress_bar_new ();
 	gtk_widget_show ((GtkWidget *) progress);
-	gtk_box_pack_start_defaults ((GtkBox *) hbox, (GtkWidget *) progress);
+	gtk_box_pack_start ((GtkBox *) hbox, (GtkWidget *) progress, TRUE, TRUE, 0);
 
 	gtk_widget_show (window);
 }
@@ -116,7 +110,7 @@ show_error_dialog()
 					       GTK_DIALOG_DESTROY_WITH_PARENT,
 					       GTK_MESSAGE_ERROR,
 					       GTK_BUTTONS_OK,
-					       err_string);
+					       "%s", err_string);
 	gtk_dialog_run (GTK_DIALOG (error_dialog));
 	gtk_widget_destroy (error_dialog);
 
@@ -294,7 +288,7 @@ cp_r (char *src, const char *dest)
 }
 
 static gchar *
-form_dir_path (char *file_name, char *delim)
+form_dir_path (char *file_name, const char *delim)
 {
 	GString *path = g_string_new (NULL);
 	gchar *dir_path;
@@ -315,7 +309,8 @@ static gchar*
 get_contacts_dir_from_filename(const char *migr_file)
 {
 	char *file_to_be_migrated = g_strdup (migr_file);
-	char *dot, *delim = "_", *file_name;
+	char *dot, *file_name;
+	const char *delim = "_";
 	gchar *dir_path = NULL;
 
 	dot = strchr (file_to_be_migrated, '.');
